@@ -33,7 +33,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
-#include <gdk/gdkx.h>
+#include <gtk/gtkx.h>
 #include <gtk/gtk.h>
 
 #include <gdk/gdk.h>
@@ -583,17 +583,14 @@ totem_embedded_launch_player (TotemEmbedded *embedded,
 	GdkScreen *screen;
 	GdkAppLaunchContext *ctx;
 	gboolean result;
-	const char *uri;
 
 	g_return_val_if_fail (embedded->app != NULL, FALSE);
 
 	if (embedded->type == TOTEM_PLUGIN_TYPE_NARROWSPACE
 		   && embedded->href_uri != NULL) {
 		uris = g_list_prepend (uris, embedded->href_uri);
-		uri = embedded->href_uri;
 	} else {
 		uris = g_list_prepend (uris, embedded->current_uri);
-		uri = embedded->current_uri;
 	}
 
 	ctx = gdk_display_get_app_launch_context (gtk_widget_get_display (embedded->window));
@@ -1715,15 +1712,6 @@ controls_size_allocate_cb (GtkWidget *controls,
 }
 #endif
 
-static void
-video_widget_size_allocate_cb (GtkWidget *controls,
-			       GtkAllocation *allocation,
-			       BaconVideoWidget *bvw)
-{
-	bacon_video_widget_set_show_visualizations (bvw, allocation->height > MINIMUM_VIDEO_SIZE);
-	g_signal_handlers_disconnect_by_func (controls, G_CALLBACK (video_widget_size_allocate_cb), NULL);
-}
-
 static void
 totem_embedded_action_volume_relative (TotemEmbedded *emb, double off_pct)
 {
@@ -1797,14 +1785,13 @@ totem_embedded_key_press_event (GtkWidget *widget, GdkEventKey *event,
 
 static gboolean
 totem_embedded_construct (TotemEmbedded *emb,
-			  GdkNativeWindow xid,
+			  Window xid,
 			  int width,
 			  int height)
 {
 	GtkWidget *child, *container, *image;
 	GtkWidget *popup_button;
 	GtkWidget *menu;
-	BvwUseType type;
 	GError *err = NULL;
 
 	emb->xml = totem_interface_load ("mozilla-viewer.ui", TRUE,
@@ -1827,18 +1814,9 @@ totem_embedded_construct (TotemEmbedded *emb,
 		emb->window = GTK_WIDGET (gtk_builder_get_object (emb->xml, "window"));
 	}
 
-	if (emb->hidden || emb->audioonly != FALSE)
-		type = BVW_USE_TYPE_AUDIO;
-	else
-		type = BVW_USE_TYPE_VIDEO;
-
-	if (type == BVW_USE_TYPE_VIDEO && emb->controller_hidden != FALSE) {
-		emb->bvw = BACON_VIDEO_WIDGET (bacon_video_widget_new
-					       (width, height, BVW_USE_TYPE_VIDEO, &err));
-	} else {
-		emb->bvw = BACON_VIDEO_WIDGET (bacon_video_widget_new
-					       (-1, -1, type, &err));
-	}
+	/* if (emb->hidden || emb->audioonly != FALSE)
+	 * FIXME disable video decoding */
+	emb->bvw = BACON_VIDEO_WIDGET (bacon_video_widget_new (&err));
 
 	/* FIXME: check the UA strings of the legacy plugins themselves */
 	/* FIXME: at least hxplayer seems to send different UAs depending on the protocol!? */
@@ -1910,16 +1888,14 @@ totem_embedded_construct (TotemEmbedded *emb,
 			  G_CALLBACK (property_notify_cb_volume), emb);
 
 	container = GTK_WIDGET (gtk_builder_get_object (emb->xml, "video_box"));
-	if (type == BVW_USE_TYPE_VIDEO) {
-		gtk_container_add (GTK_CONTAINER (container), GTK_WIDGET (emb->bvw));
-		/* FIXME: why can't this wait until the whole window is realised? */
-		gtk_widget_realize (GTK_WIDGET (emb->bvw));
-		gtk_widget_show (GTK_WIDGET (emb->bvw));
 
-		/* Let us know when the size has been allocated for the video widget */
-		g_signal_connect_after (G_OBJECT (emb->bvw), "size-allocate",
-					G_CALLBACK (video_widget_size_allocate_cb), emb->bvw);
-	} else if (emb->audioonly != FALSE) {
+	gtk_container_add (GTK_CONTAINER (container), GTK_WIDGET (emb->bvw));
+	/* FIXME: why can't this wait until the whole window is realised? */
+	gtk_widget_realize (GTK_WIDGET (emb->bvw));
+	gtk_widget_show (GTK_WIDGET (emb->bvw));
+
+	/* FIXME disable video decoding when hidden */
+	if (emb->audioonly != FALSE) {
 		gtk_widget_hide (container);
 	}
 
@@ -2086,7 +2062,7 @@ totem_embedded_set_window (TotemEmbedded *embedded,
 	embedded->width = width;
 	embedded->height = height;
 
-	totem_embedded_construct (embedded, (GdkNativeWindow) window,
+	totem_embedded_construct (embedded, (Window) window,
 				  width, height);
 
 	return TRUE;
@@ -2273,6 +2249,8 @@ static GOptionEntry option_entries [] =
 
 int main (int argc, char **argv)
 {
+	GOptionContext *context;
+	GOptionGroup *baconoptiongroup;
 	TotemEmbedded *emb;
 	DBusGProxy *proxy;
 	DBusGConnection *conn;
@@ -2344,9 +2322,15 @@ int main (int argc, char **argv)
 	}
 #endif
 
-	if (!gtk_init_with_args (&argc, &argv, NULL, option_entries, GETTEXT_PACKAGE, &e))
-	{
-		g_print ("%s\n", e->message);
+	context = g_option_context_new ("- Play audio and video inside a web browser");
+	baconoptiongroup = bacon_video_widget_get_option_group();
+	g_option_context_add_main_entries (context, option_entries, GETTEXT_PACKAGE);
+	g_option_context_set_translation_domain(context, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, baconoptiongroup);
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+
+	if (g_option_context_parse (context, &argc, &argv, &e) == FALSE) {
+		g_print ("Failed to parse options: %s\n", e->message);
 		g_error_free (e);
 		exit (1);
 	}
@@ -2362,8 +2346,6 @@ int main (int argc, char **argv)
 		exit (1);
 	}
 
-	bacon_video_widget_init_backend (NULL, NULL);
-
 	dbus_g_object_type_install_info (TOTEM_TYPE_EMBEDDED,
 					 &dbus_glib_totem_embedded_object_info);
 	g_snprintf (svcname, sizeof (svcname),
